%% Clear Variables and Open FUTEK USB DLL Assembly

clear variables;
pause('on');

%Net.addAssembly requires the full file path to access private .dll files
%such as the FUTEK USB DLL.
%With the current folder set as where both this script and the .dll file
%are located, the which() function returns the full file path.

%The included copy of FUTEK_USB_DLL.dll has been compiled for either 32 bit
%or 64 bit platforms. If you are running MATLAB 32 bit use only the 32 bit
%.dll file and if you are using MATLAB 64 bit use only the 64 bit .dll
%file.

%Full .dll file API available here: http://www.futek.com/files/docs/API/FUTEK_USB_DLL/webframe.html

%Requires .Net 4.0 and Windows
NET.addAssembly(which('FUTEK_USB_DLL.dll'));
import FUTEK_USB_DLL.*
import FUTEK_USB_DLL.USB_DLL.*

%Serial Number in ' ' for your Instrument or USB Output Kit
serialNumber = '660'; 

%% Initialize Variables

instr = USB_DLL();
samples = 100;
deviceHandle = '';
deviceStatus = 0; %#ok<NASGU>
deviceStatusChr = '';
temp = 0;
offsetValue = 0;
fullScaleValue = 0;
fullScaleLoad = 0;
decimalPoint = 0;
normalData = 0;
calculatedReading = 0;
unitCode = 0;
unitChr = '';
unitCodeData = {'atm'; 'bar'; 'dyn'; 'ft-H20'; 'ft-lb'; 'g'; 'g-cm'; 'g-mm'; 'in-H20'; 'in-lb';
    'in-oz'; 'kdyn'; 'kg'; 'kg-cm'; 'kg/cm2'; 'kg-m'; 'klbs'; 'kN'; 'kPa'; 'kpsi';
    'lbs'; 'Mdyn'; 'mmHG'; 'mN-m'; 'MPa'; 'MT'; 'N'; 'N-cm'; 'N-m'; 'N-mm';
    'oz'; 'psi'; 'Pa'; 'T'; 'mV/V'; 'A'; 'mA'; 'A'; 'mm'; 'cm';
    'dm'; 'm'; 'km'; 'in'; 'ft'; 'yd'; 'mi'; 'g'; 'mg'; 'LT';
    'mbar'; '?C'; '?F'; 'K'; '?Ra'; 'kN-m'; 'g-m'; 'nV'; 'V'; 'mV';
    'V'; 'kV'; 'NONE'};
calculatedTable = cell(samples,4);

%% Open Connection and Begin Data Capture Run

%Intializes the connection to instrument with the specified Serial Number.
%Fetches Device Status to determine if connection is successful or returns
%the error code and exits
instr.Open_Device_Connection(serialNumber);
deviceStatus = instr.DeviceStatus();
if deviceStatus == 0
    disp('Pass')
else
    deviceStatusChr = num2str(deviceStatus);
    disp(['Device Error ' deviceStatusChr])
    return;
end;

%Fetches the device handle needed to retrieve stored values used in load
%calculation
deviceHandle = instr.DeviceHandle();

%Begin loop to collect samples
for x=1:samples
    %% Capture Variables
    
    %Uses the FUTEK USB DLL to retrieve values for load calculation
    %The .dll returns these values as a System.String
    %This is then converted to a char value and then a number
    %Lastly if there was a retrieval error the .dll returns 'Error' which
    %converts to an empty value. isEmpty() test for that and continues
    %retrieval until a number is returned.
    
    % Offset Value
    while true
        temp = str2num(char(instr.Get_Offset_Value(deviceHandle))); %#ok<*ST2NM>
        if ~(isempty(temp))
            break
        end
    end
    offsetValue = temp;
    
    %Full Scale Value
    while true
        temp = str2num(char(instr.Get_Fullscale_Value(deviceHandle)));
        if ~(isempty(temp))
            break
        end
    end
    fullScaleValue = temp;
    
    % Full Scale Load
    while true
        temp = str2num(char(instr.Get_Fullscale_Load(deviceHandle)));
        if ~(isempty(temp))
            break
        end
    end
    fullScaleLoad = temp;
   
    % Decimal Point
    while true
        temp = str2num(char(instr.Get_Decimal_Point(deviceHandle)));
        if ~(isempty(temp))
            break
        end
    end
    decimalPoint = temp;
    %Test to see if decimal point returned is outside defined decimal point
    %values
    %See here for more information: 
    %http://www.futek.com/files/docs/API/FUTEK_USB_DLL/webframe.html#DecimalPointCodes.html
    if decimalPoint >= 6
        disp(['Decimal Point Out of Range/Undefined']) 
        return;
    end
    
    % Get Unit Code and Convert to Actual Load
    while true
        temp = str2num(char(instr.Get_Unit_Code(deviceHandle)));
        if ~(isempty(temp))
            break
        end
    end
    unitCode = temp + 1;
    unitChr = unitCodeData{unitCode,1};
   
    %Get Normal Data
    while true
        temp = str2num(char(instr.Normal_Data_Request(deviceHandle)));
        if ~(isempty(temp))
            break
        end
    end
    normalData = temp;
    
    %% Calculate Read Out and Display
    
    % Use retrieved variables to calculate measured load cell output
    calculatedReading = normalData - offsetValue;
    calculatedReading = calculatedReading/(fullScaleValue - offsetValue);
    calculatedReading = calculatedReading*fullScaleLoad;
    calculatedReading = calculatedReading/10^decimalPoint;
    tnow = datestr(now,'dd-mm-yyyy HH:MM:SS FFF'); %Retrieve Current Time
    
    dispStr = [num2str(calculatedReading), ' ', unitChr];
    disp(dispStr)
    
    %Store Sample Number, Reading, Units, and Time in a Cell Array
    calculatedTable(x,1:4)={x, calculatedReading, unitChr, tnow};
    
    %Pause program to prevent repolling the instrument
    pause(0.01);
end
%% Close Connection

% Close the connection to properly disconnect device and allow access by
% other programs, otherwise you must disconnect drive from USB port.
instr.Close_Device_Connection(deviceHandle);